#ifndef _PROTO_H_
#define _PROTO_H_

#include <arc/types.h>
#include <arc/errno.h>
#include <arc/boot.h>
#include <arc/configuration.h>
#include <arc/memory.h>
#include <arc/misc.h>
#include <arc/pathname.h>

#ifndef BIOS
#define BIOS(offset,index,returntype,name,params) returntype name params;
#endif

BIOS(0x0,1,_STATUS,Load,
        (PCHAR ImagePath,
        ULONG TopAddress,
        PULONG EntryAdress,
        PULONG LowAddress))
BIOS(0x4,2,_STATUS,Invoke,
        (ULONG EntryAddress,
        ULONG StackAddress,
        ULONG Argc,
        PCHAR Argv[],
        PCHAR Envp[]))
BIOS(0x8,3,_STATUS,Execute,
        (PCHAR ImagePatch,
        ULONG Argc,
        PCHAR Argv[],
        PCHAR Envp[]))
BIOS(0xc,4,VOID,Halt,(VOID))
BIOS(0x10,5,VOID,PowerDown,(VOID))
BIOS(0x14,6,VOID,Restart,(VOID))
BIOS(0x18,7,VOID,Reboot,(VOID))
BIOS(0x1c,8,VOID,EnterInteractiveMode,(VOID))

/*
 * Function: (unused)
 * Offset  : 0x20
 * Index   : 9
 */

BIOS(0x24,10,PCONFIGURATION_COMPONENT,GetPeer,
        (PCONFIGURATION_COMPONENT Component))
BIOS(0x28,11,PCONFIGURATION_COMPONENT,GetChild,
        (PCONFIGURATION_COMPONENT Component))
BIOS(0x2c,12,PCONFIGURATION_COMPONENT,GetParent,
        (PCONFIGURATION_COMPONENT Component))
BIOS(0x30,13,_STATUS,GetConfigurationData,(PVOID ConfigurationData,
        PCONFIGURATION_COMPONENT Component))
BIOS(0x34,14,PCONFIGURATION_COMPONENT,AddChild,
        (PCONFIGURATION_COMPONENT Component,
        PCONFIGURATION_COMPONENT NewComponent,
        PVOID ConfigurationData))
BIOS(0x38,15,_STATUS,DeleteComponent,
        (PCONFIGURATION_COMPONENT Component))
BIOS(0x3c,16,PCONFIGURATION_COMPONENT,GetComponent,
        (PCHAR Path))     
BIOS(0x40,17,_STATUS,SaveConfiguration,(VOID))
BIOS(0x44,18,PSYSTEM_ID,GetSystemId,(VOID))
BIOS(0x48,19,PMEMORY_DESCRIPTOR,GetMemoryDescriptor,
        (PMEMORY_DESCRIPTOR MemoryDescriptor))   

/*
 * Function: (unused)
 * Offset  : 0x4c
 * Index   : 20
 */

BIOS(0x50,21,PTIME_FIELDS,GetTime,(VOID))
BIOS(0x54,22,ULONG,GetRelativeTime,(VOID))
BIOS(0x58,23,_STATUS,GetDirectoryEntry,
        (ULONG FileId,
        PDIRECTORY_ENTRY Buffer,
        ULONG Length,
        PULONG Count))
BIOS(0x5c,24,_STATUS,Open,
        (PCHAR OpenPath,
        OPEN_MODE OpenMode,
        PULONG FileId))
BIOS(0x60,25,_STATUS,Close,
        (ULONG FileId))
BIOS(0x64,26,_STATUS,Read,
        (ULONG FileId,
        PVOID Buffer,
        ULONG Length,
        PULONG Count))
BIOS(0x68,27,_STATUS,GetReadStatus,
        (ULONG FileId))
BIOS(0x6c,28,_STATUS,Write,
        (ULONG FileId,
        PVOID Buffer,
        ULONG Length,
        PULONG Count))
BIOS(0x70,29,_STATUS,Seek,
        (ULONG FileId,
        PLARGE_INTEGER Offset,
        SEEK_MODE SeekMode))
BIOS(0x74,30,_STATUS,Mount,
        (PCHAR MountPath,
        MOUNT_OPERATION Operation))
BIOS(0x78,31,PCHAR,GetEnvironmentVariable,
        (PCHAR Variable))
BIOS(0x7c,32,_STATUS,SetEnvironmentVariable,
        (PCHAR Variable,
        PCHAR Value))
BIOS(0x80,33,_STATUS,GetFileInformation,
        (ULONG FileId,
        PFILE_INFORMATION FileInformation))
BIOS(0x84,34,_STATUS,SetFileInformation,
        (ULONG FileId,
        ULONG AttributeFlags,
        ULONG AttributeMask))
BIOS(0x88,35,VOID,FlushAllCaches,(VOID))
BIOS(0x8c,36,_STATUS,TestUnicodeCharacter,
        (ULONG FileId,
        WCHAR UnicodeCharacter))
BIOS(0x90,37,P_DISPLAY_STATUS,GetDisplayStatus,
        (ULONG FileId))

#endif /* _PROTO_H_ */
